gizmos = (
    ('gizmo_test_tags', 'test_simple_tag', 'slot_name'),
    ('gizmo_test_tags', 'test_simple_tag', 'another_slot_name', \
            ['url_name', ]),
    ('gizmo_test_tags', 'test_simple_tag', 'another_slot_name', \
            ['another_name', ]),
)
